
! (c) sparslab module name=rif_ls_right_main
!
! purpose:
!     symmetric- right-looking RIF for the least squares problem
!     a sample driver
!
      use rif_mod
!

      integer n,i
      double precision zero
      parameter(zero=0.0d0)
      integer msglvl,msgunit,size_r,size_c,size_l
      integer, allocatable :: il(:),jl(:)
      double precision, allocatable :: al(:)
      integer garcol,garrow,droptyp
      integer imodif,diag_one,filldyn,fillmax,ifillmax,info
      integer fillic,nja,m,ierr,nrealloc,nzata
      double precision drflic,drflai,diagtol,shift
      integer, allocatable, dimension(:) :: ia,ja,wn01
      double precision, allocatable, dimension(:) :: aa,x,y
      integer, allocatable, dimension(:) :: iat,jat
      double precision, allocatable, dimension(:) :: aat
!
! start of rif_ls_right_main
!
!  -- generate input test matrix in csr format
!
      nx=17
      n=nx*nx
      nja=(4*2+(nx-2)*(nx-2)*4+4*(nx-2)*3)+n
      m=n
      allocate(ia(m+1),stat=ierr)
      allocate(ja(nja),aa(nja),stat=ierr)
      call laplace(nx,ia,ja,aa)
!
!  -- transpose the input matrix
!
      allocate(wn01(m+1),stat=ierr)
      nja=ia(m+1)-1
      allocate(iat(n+1),jat(nja),aat(nja),stat=ierr)
      call trancs1(m,n,ia,ja,aa,iat,jat,aat,wn01)
      deallocate(wn01)
!
!  -- get sample right-hand side
!
      allocate(x(m),y(m))
!
      do i=1,n
        y(i)=dble(i)
      end do
      call mvucsr1(m,ia,ja,aa,y,x)
      call mvucsr1(n,iat,jat,aat,x,y)
!
!
!     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!  -- start of parameters to be set by user
!     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!
!  -- set the drop tolerances
!  -- a/ for the final rif factor
!
      drflic=0.0d0
!
!  -- b/ for the sainv processs
!
      drflai=0.0d0
!
!     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!  -- end of parameters to be set by user
!     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!
!
!     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!  -- start of defaults -- not needed to change
!     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!
!  -- sizes: initial guess
!
      size_l=1000000
      size_r=1000000
      size_c=1000000
!
!  -- other parameters without the need to be changed
!
      diagtol=1.1d-16
      droptyp=0
      diag_one=1
      nzata=0 ! dummy value - used only in debug mode
      shift=0.0d0
!
!     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!  -- end of defaults -- need not to be changed
!     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!
!
!  -- standard level of output
!
      msglvl=2
!
!  -- open unit for output
!
      msgunit=2
      open(unit=msgunit,file='output.out')
!
!  -- call RIF
!
      call rifsrnr0(msglvl,msgunit,m,n,ia,ja,aa,
     *  iat,jat,aat,size_c,
     *  size_r,il,jl,al,size_l,
     *  diagtol,drflic,drflai,shift,diag_one,droptyp,nrealloc,imodif,
     *  filldyn,fillmax,
     *  ifillmax,garrow,garcol,fillic,nzata,info)
      if(info /= 0) go to 400
!
      deallocate(iat,jat,aat)
!
!  -- output preconditioner info
!
      write(*,*)' preconditioner successfully computed'
      write(2,*)' preconditioner successfully computed'
      write(2,*)' preconditioner size',fillic
      write(2,*)' allocatables'
      write(2,*) il(1:n+1)
      write(2,*)' column indices'
      write(2,*) jl(1:il(n+1)-1)
      write(2,*)' nonzero entries'
      write(2,*) al(1:il(n+1)-1)
!
!  -- solve the system of normal equations
!
      call bkcch(n,il,jl,al,y,x,0)
!
!  -- print solution x
!
      write(2,*)' solution x'
      write(2,*) x(1:n)
!
!  -- we have the result in y
!
      deallocate(x,y)
!
      close(2)
      stop ' success'
 400  continue
      stop ' some error'
     
!
! end of rif_ls_right_main
!
      end
